CControlBase@ ControlFromID(CControlContainer@ container, const string &in id)
{
	if (container is null) {
		return null;
	}

	for (uint i = 0; i < container.Childs.Length; i++) {
		auto child = container.Childs[i];
		if (child.IdName == id) {
			return child;
		}
	}
	return null;
}

bool g_visible = false;

[Setting name="Squish"]
bool g_squish = false;

[Setting name="Alpha" min=0 max=1]
float g_alpha = 0.9f;

UI::Texture@ g_mask;

void Render()
{
	if (!g_visible) {
		return;
	}

	float screenWidth = float(Draw::GetWidth());
	float screenHeight = float(Draw::GetHeight());

	const float startFactorX = 0.0185f;
	const float startFactorY = 0.214f;
	const float sizeFactorX = 0.113f;
	const float sizeFactorY = 0.2003f;

	float startX = startFactorX * screenWidth;
	float startY = startFactorY * screenHeight;
	float w = sizeFactorX * screenWidth;
	float h = sizeFactorY * screenHeight;

	int windowFlags = UI::WindowFlags::NoSavedSettings;
	windowFlags |= UI::WindowFlags::NoTitleBar;
	windowFlags |= UI::WindowFlags::NoResize;
	windowFlags |= UI::WindowFlags::NoMove;
	UI::SetNextWindowPos(int(startX) - 10, int(startY + h) + 20, UI::Cond::Always);
	UI::SetNextWindowSize(int(w + 20), 0, UI::Cond::Always);
	UI::PushStyleColor(UI::Col::WindowBg, vec4(0.007f, 0.231f, 0.122f, 0.8f));
	if (UI::Begin("Thumbnail", windowFlags)) {
		g_squish = UI::Checkbox("Squish", g_squish);
		UI::SameLine();
		UI::SetNextItemWidth(UI::GetContentRegionAvail().x);
		g_alpha = UI::SliderFloat("", g_alpha, 0.0f, 1.0f);
	}
	UI::End();
	UI::PopStyleColor();

	if (g_squish) {
		const float squishFactor = 0.7222f;
		startY += h * (1.0f - squishFactor) / 2.0f;
		h *= squishFactor;
	}

	uint8 alpha = uint8(g_alpha * 255.0f);

	auto dl = UI::GetBackgroundDrawList();
	dl.AddImage(g_mask, vec2(startX, startY), vec2(w, h), 0x023B1F00 | alpha);
}

void Main()
{
	@g_mask = UI::LoadTexture("mask.png");

	while (true) {
		g_visible = false;

		auto app = GetApp();
		auto editor = cast<CGameCtnEditorFree>(app.Editor);
		if (editor !is null) {
			auto snapCamera = ControlFromID(editor.EditorInterface.InterfaceRoot, "FrameEditSnapCamera");
			if (snapCamera !is null) {
				g_visible = !snapCamera.IsHiddenExternal;

				auto snapView = ControlFromID(cast<CControlContainer>(snapCamera), "ButtonSnap");
				if (snapView !is null) {
					bool update = false;
					float updateY = 0.0f;

					if (g_squish && snapView.BoxMax.y > 0.08f) {
						update = true;
						updateY = 0.08f;
					} else if (!g_squish && snapView.BoxMax.y < 0.18f) {
						update = true;
						updateY = 0.18f;
					}

					if (update) {
						snapView.BoxMax = vec2(0.18f, updateY);
						snapView.IsHiddenExternal = true;
						snapView.IsVisible = true;
					}
				}
			}
		}

		yield();
	}
}
